import sys
import os
import re
from datetime import date
from lib.pdfminer.pdfminer.pdfparser import PDFParser
from lib.pdfminer.pdfminer.pdfdocument import PDFDocument
from lib.tika.tika import parser
from lib.langdetect.langdetect import detect
from lib.langdetect.langdetect.lang_detect_exception import LangDetectException
os.environ['TIKA_SERVER_JAR'] = 'https://repo1.maven.org/maven2/org/apache/tika/tika-server/1.19/tika-server-1.19.jar'


def replaceNewlines(txt):
    tmp = txt.split("\n")
    res = ""
    for line in tmp:
        if line.replace(" ", "") == "":
            continue
        if res != "":
            res += '\n'
        res += line
    return res


def getTextFromPDF(fileAddress):
    parsed_pdf = parser.from_file(fileAddress)
    data = parsed_pdf['content']
    return replaceNewlines(data)


def detectLanguages(abstract, languages):
    if re.search('[a-zA-Z]', abstract) == None:
        return languages
    try:
        tmp = detect(abstract)
        if tmp in languages:
            languages[tmp] += 1
        else:
            languages[tmp] = 1
    except LangDetectException:
        pass
    return languages


def estimateLanguagesPercentage(languages):
    total = 0
    max = 0
    principal = ""
    res = {}
    for l in languages:
        total += int(languages[l])
    for l in languages:
        tmp = int(languages[l]) / total * 100
        if tmp > max:
            max = tmp
            principal = l
        res[l] = tmp
    return res, principal


def detectMultipleLanguages(languages, source):
    languages, principal = estimateLanguagesPercentage(languages)
    detected = ""
    for l in languages:
        if languages[l] >= 15:
            if detected != "":
                detected += " "
            detected += l
    if len(detected) > 2:
        print("Warning ! Multiple languages have been detected at the source :  \"" + source + "\" !")
        print("The following languages have been detected : " + detected)
        print("This could massively affect the analysis of Gargantext.")
        print("You can translate it with the TsvTranslator tool.")

    return principal

def detectMultiplePdfLanguages(pdfLanguages):
    languages = []
    for l in pdfLanguages.values():
        if l not in languages and len(languages) == 1:
            print("Warning ! Multiple languages have been detected for your pdf files !")
            print("The following languages have been detected : " + str(pdfLanguages))
            print("This could massively affect the analysis of Gargantext.")
            print("You can translate them with the TsvTranslator tool.")
            return
        if len(languages) == 0:
            languages.append(l)

def removeFiligrane(abstract, filigrane):
    if filigrane == "":
        return abstract.split("\n")
    tmp = abstract.split("\n")
    txt = "".join(tmp).replace(filigrane, "")
    res = reSplit(txt, tmp)
    return res


def reSplit(new, old):
    res = []
    count = 0
    for s in old:
        tmp = ""
        for c in s:
            if c == new[count]:
                tmp += c
                count += 1
        if (tmp != ""):
            res.append(tmp)
    return res


def segmentAbstract(fileAddress, tsv, author, source, year, month, day, filigrane, pdfLanguages):
    abstract = getTextFromPDF(fileAddress)
    abstract = removeFiligrane(abstract, filigrane)
    nbLines = len(abstract)
    n = 0
    count = 1
    languages = {}
    while n < nbLines - 2:
        doc = "\n".join(abstract[n: n + 9]).replace("�", "")
        title = source + " : Part " + str(count)
        tsv += correctedSequence(author, False) + "\t" + correctedSequence(
            source, False) + "\t" + year + "\t" + month + "\t" + day + "\t"
        tsv += correctedSequence(title, False) + "\t"
        tsv += correctedSequence(doc, True)
        if tsv[-1] != "\n":
            tsv += "\n"
        n += 7
        count += 1
        if n > nbLines - 9 and n != nbLines - 2:
            n = nbLines - 9
        languages = detectLanguages(doc, languages)

    principal = detectMultipleLanguages(languages, source)
    pdfLanguages[source] = principal
    return tsv, pdfLanguages


def getDate(fileAddress):
    year = str(date.today().year)
    month = "1"
    day = "1"
    with open(fileAddress, 'rb') as fp:
        parser = PDFParser(fp)
        doc = PDFDocument(parser)
        info = doc.info[0]
        tmp = ""
        if "CreationDate" in info:
            tmp = info["CreationDate"].decode('utf-8')
            year = tmp[2:6]
            month = tmp[6:8]
            day = tmp[8:10]

    return year, month, day


def waitAnswer(question):
    answer = input(question + " (Y/N) : ")
    while (answer.lower() != "y" and answer.lower() != "n"):
        answer = input("Wrong input ! Answer with either y or n : ")

    return answer == "y"


def getAuthorAndTitle(fileName,  inspect):

    author = "null"
    title = fileName.replace(".pdf", "")

    if (inspect):
        question = "The author isn't set for the file \"" + \
            fileName + "\". Do you want to set it ?"
        if waitAnswer(question):
            author = input("Please enter the author(s) name : ")

        question = "Do you want to change the title for \"" + fileName + "\" ?"
        if waitAnswer(question):
            title = input("Please enter the title : ")
    return author, title


def correctedSequence(text, last):
    tmp = text.replace("\"", "\"\"")
    find = "\t" in text or "\"" in text or "\n" in text
    if find:
        if text[len(text) - 1] == "\n":
            tmp = tmp[:-1]
        tmp = "\"" + tmp + "\""
        if last:
            tmp += "\n"
    return tmp


def getFiligrane(fileName):
    filigrane = ""
    question = "Does the file \"" + fileName + "\" has a filigrane ?"
    if waitAnswer(question):
        filigrane = input("Please enter the filigrane : ")
    return filigrane


def txtToTSV(fileName, fileAddress, inspect, pdfLanguages):
    tsv = "authors\tsource\tpublication_year\tpublication_month\tpublication_day\ttitle\tabstract\n"
    year, month, day = getDate(fileAddress)
    author, source = getAuthorAndTitle(fileName, inspect)
    filigrane = getFiligrane(fileName)
    print("Processing conversion of pdf : " + fileName)
    tsv, pdfLanguages = segmentAbstract(
        fileAddress, tsv, author, source, year, month, day, filigrane, pdfLanguages)
    with open("tsv/" + fileName.replace(".pdf", ".tsv"), "w", encoding="utf-8-sig") as file:
        file.write(tsv)

    tsv = "\n".join(tsv.split("\n")[1:])

    return tsv, pdfLanguages


def convertAllPDF(directoryName):

    question = "Do you want to change authors and titles for each pdf ?"
    inspect = waitAnswer(question)
    tsv = "authors\tsource\tpublication_year\tpublication_month\tpublication_day\ttitle\tabstract\n"
    pdfLanguages = {}
    for file in os.listdir(directoryName):
        if ".pdf" in file:
            tmp, pdfLanguages = txtToTSV(
                file, directoryName + "/" + file, inspect, pdfLanguages)
            tsv += "\n" + tmp

    detectMultiplePdfLanguages(pdfLanguages)

    with open("tsv/final.tsv", "w", encoding='utf-8-sig') as file:
        file.write(tsv)

    print("Warning ! Be aware that, depending on your pdf, the extracted text can have some modifications.")
    print("In case you want to modify it manually, you can use the PDFtoTXT and TXTtoPDF tools.")


def main():

    if len(sys.argv) == 2:
        fileName = sys.argv[1]
        directoryName = sys.argv[1]
        convertAllPDF(directoryName)

    return


if __name__ == '__main__':
    sys.exit(main())
